@@ -4,16 +4,16 @@ package Finance::Card::Citibank;
 # Finance::Card::Citibank
 # Mark Grimes
 #
-# Check you credit card balances.
-# Copyright (c) 2005 Mark Grimes (mgrimes@cpan.org).
+# Check your credit card balances.
+# Copyright (c) 2005-8 Mark Grimes (mgrimes@cpan.org).
 # All rights reserved. This program is free software; you can redistribute
 # it and/or modify it under the same terms as Perl itself.
 #
-# Formatted with tabstops at 4
+# Parts of this package were inspired by Simon Cozens'
+# Finance::Bank::Lloyds module. Thanks Simon!
 #
-# Parts of this package were inspired by:
-#   Simon Cozens - Finance::Bank::Lloyds module
-# Thanks!
+# Jon Keller contributed much of the code to work with multiple accounts.
+# Thanks Jon!
 #
 ###########################################################################
 
@@ -23,59 +23,64 @@ use warnings;
 use Carp;
 use WWW::Mechanize;
 
-our $VERSION = '1.64';
-
-our $ua = WWW::Mechanize->new(
-    env_proxy => 1, 
-    keep_alive => 1, 
-    timeout => 30,
-); 
+our $VERSION = '1.70';
 
+my $ua = WWW::Mechanize->new(
+    env_proxy  => 1,
+    keep_alive => 1,
+    timeout    => 30,
+);
 
 sub check_balance {
-    my ($class, %opts) = @_;
-    my $self = bless { %opts }, $class;
+    my ( $class, %opts ) = @_;
+    my $self = bless {%opts}, $class;
     my $content;
 
-    if( $opts{content} ){
+    if ( $opts{content} ) {
+
         # If we give it a file, use the file rather than downloading
-        local $/;
-        open my $fh, "<", $opts{content} or die;
-        $content = <$fh>;
+        open my $fh, "<", $opts{content} or confess;
+        $content = do { local $/ = undef; <$fh> };
         close $fh;
+
     } else {
         croak "Must provide a password" unless exists $opts{password};
         croak "Must provide a username" unless exists $opts{username};
 
-        $ua->get("http://www.citicards.com/cards/wv/home.do") 
-            or die "couldn't load the initial page";
+        $ua->get("http://www.citicards.com/cards/wv/home.do")
+          or confess "couldn't load the initial page";
 
         $ua->submit_form(
-            form_name 	=> 'LOGIN',
-            fields		=> {
-                    'USERNAME'	    => $opts{username},
-                    'PASSWORD'	    => $opts{password},
-                    'NEXT_SCREEN'   => '/AccountSummary',
-                },
-        ) or die "couldn't submit the login form";
+            form_name => 'LOGIN',
+            fields    => {
+                'USERNAME'    => $opts{username},
+                'PASSWORD'    => $opts{password},
+                'NEXT_SCREEN' => '/AccountSummary',
+            },
+        ) or confess "couldn't submit the login form";
 
-        $ua->submit_form(
-            form_number => 2,
-        ) or die "couldn't submit the account selection form";
         $content = $ua->content;
-    }
-	
-	if( $opts{log} ){
-        # Dump to the filename passed in log
-		open( my $fh,">", $opts{log});
-		print $fh $ua->content;
-		close $fh;
-	}
 
-    # TODO: deal with multiple accounts... seems to use js to get other accounts?
-	my @accnts = $content =~
-		m!
-          <span\sclass="prodName"><a\sname="view1c"></a>(.*?)</span></td>\s*
+        # First account's data is in content, subsequent accounts' data are
+        # ajax loaded from .../GetDashboardAccount. Concatenate the two pages
+        # and search them together
+
+        $ua->get("https://www.accountonline.com/GetDashboardAccounts")
+          or confess "couldn't load GetDashboardAccounts";
+        $content .= $ua->content;
+    }
+
+    if ( $opts{log} ) {
+
+        # Dump to the filename passed in log
+        open( my $fh, ">", $opts{log} ) or confess;
+        print $fh $content;
+        close $fh;
+    }
+
+    # Extract the relevant info into one array
+    my @accnts = $content =~ m!
+          <span\sclass="prodName"><a\sname="View(\d+)c"></a>([^\n]+?)</span></td>\s*
           </tr>\s*
           <tr>\s*
           <td>&nbsp;(Account\sending\sin:\s*\d+)</td>\s*
@@ -85,42 +90,46 @@ sub check_balance {
           <tr>\s*
           <td\s[^>]*><div[^>]*><span\sclass="balNdue">\$([\d,\.]+)</span></div></td>
      !xisg;
-	warn "couldn't find any accounts" unless @accnts;
+    carp "couldn't find any accounts" unless @accnts;
 
-	my @accounts;
-    while( @accnts ){
-        my ($name, $account_no, $balance)
-                = ( shift @accnts, shift @accnts, shift @accnts );
+    my @accounts;
+    while (@accnts) {
+        my ( $position, $name, $account_no, $balance ) = splice @accnts, 0, 4;
 
         $name       =~ s/&[^;]*;//g;
         $name       =~ s/<[^>]*>//g;
         $name       =~ s/\s+/ /g;
         $account_no =~ s/\s+/ /g;
         $balance    =~ s/,//g;
-        $balance    *= -1;
-	
-        # warn "# Name: $name\n";
-        # warn "# Account: $account_no\n";
-        # warn "# Balance: $balance\n";
+        $balance *= -1;
 
-        push @accounts, (bless {
-            balance		=> $balance,
-            name		=> $name,
-            sort_code	=> $account_no,
-            account_no	=> $account_no,
-            # parent		=> $self,
-            statement	=> undef,
-        }, "Finance::Card::Citibank::Account");
+        # carp "# Position: $position\n";  # i.e. "1" for the 1st account..."n" for the nth account
+        # carp "# Name: $name\n";
+        # carp "# Account: $account_no\n";
+        # carp "# Balance: $balance\n";
+
+        push @accounts, (
+
+            bless {
+                balance    => $balance,
+                name       => $name,
+                sort_code  => $account_no,
+                account_no => $account_no,
+                position  => $position, # redundant since just = array index + 1
+                statement => undef,
+                ## parent => $self,
+            },
+            "Finance::Card::Citibank::Account"
+        );
     }
 
     return @accounts;
 }
 
 package Finance::Card::Citibank::Account;
-# Basic OO smoke-and-mirrors Thingy
-# no strict; 
-our $AUTOLOAD;
-sub AUTOLOAD { my $self=shift; $AUTOLOAD =~ s/.*:://; $self->{$AUTOLOAD} }
+use base qw(Class::Accessor::Fast);
+__PACKAGE__->mk_accessors(
+    qw(balance name sort_code account_no position statement));
 
 1;
 
@@ -136,13 +145,13 @@ Finance::Card::Citibank - Check your Citigroup credit card accounts from Perl
 
   use Finance::Card::Citibank;
   my @accounts = Finance::Card::Citibank->check_balance(
-	  username => "xxxxxxxxxxxx",
-	  password => "12345",
+      username => "xxxxxxxxxxxx",
+      password => "12345",
   );
 
   foreach (@accounts) {
-	  printf "%20s : %8s / %8s : USD %9.2f\n",
-	  $_->name, $_->sort_code, $_->account_no, $_->balance;
+      printf "%20s : %8s / %8s : USD %9.2f\n",
+      $_->name, $_->sort_code, $_->account_no, $_->balance;
   }
   
 =head1 DESCRIPTION
@@ -150,8 +159,7 @@ Finance::Card::Citibank - Check your Citigroup credit card accounts from Perl
 This module provides a rudimentary interface to Citigroup online
 at C<https://www.citibank.com/us/cards/index.jsp>. 
 You will need either C<Crypt::SSLeay> or C<IO::Socket::SSL> installed 
-for HTTPS support to work. C<WWW::Mechanize> is required.
-
+for HTTPS support to work. C<WWW::Mechanize> is required.  
 =head1 CLASS METHODS
 
 =head2 check_balance()
@@ -190,11 +198,7 @@ to me, but is provided under B<NO GUARANTEE>, explicit or implied.
 Simon Cozens for C<Finance::Bank::LloydsTSB>. The interface to this module,
 some code and the pod were all taken from Simon's module.
 
-=head1 TODO
-
-Currently, only the first account is picked up. It is an easy fix (I think)
-to grab other accounts, but I don't need it. If someone out there does need
-multiple accounts, let me know and I will implement it.
+Jon Keller added the ability to pull multiple accounts.
 
 =head1 AUTHOR
 
@@ -202,7 +206,7 @@ Mark Grimes <mgrimes@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005-7 by mgrimes
+Copyright (C) 2005-8 by mgrimes
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.2 or,
